@@ -9,9 +9,9 @@ import unittest
 from collections import deque
 import numpy as np
 from PyFin.Enums import Factors
-from PyFin.Analysis.SecurityValueHolders import SecurityValueHolder
 from PyFin.Analysis.SecurityValueHolders import SecuritiesValues
 from PyFin.Analysis.SecurityValueHolders import dependencyCalculator
+from PyFin.Analysis.TechnicalAnalysis import SecurityLatestValueHolder
 from PyFin.Analysis.TechnicalAnalysis import SecurityMovingAverage
 from PyFin.Analysis.TechnicalAnalysis import SecurityMovingMax
 from PyFin.Analysis.TechnicalAnalysis import SecurityMovingSum
@@ -35,13 +35,6 @@ class TestSecurityValueHolders(unittest.TestCase):
                                                                                             calculated[name]))
         self.checker = check_values
 
-    def testSecuritiesValuesIncompatibleSymbolList(self):
-        values = SecuritiesValues({'AAPL': 3.0, 'IBM': 2.0, 'GOOG': 4.0})
-        benchmarkValues3 = {'AAPL': 3.0, 'IBM': 2.0}
-        with self.assertRaises(ValueError):
-            values3 = SecuritiesValues(benchmarkValues3)
-            _ = values + values3
-
     def testSecuritiesValuesComparison(self):
 
         benchmarkValues = SecuritiesValues({'AAPL': 1.0, 'IBM': 2.0, 'GOOG': 3.0})
@@ -158,7 +151,7 @@ class TestSecurityValueHolders(unittest.TestCase):
         symbolList = ['aapl', 'ibm']
         testValueHolder = SecurityMovingAverage(window, pNames, symbolList)
         dependency = {
-            name: pNames[0] for name in symbolList
+            name: pNames for name in symbolList
             }
 
         self.assertEqual(set(testValueHolder.symbolList), set(symbolList))
@@ -209,29 +202,30 @@ class TestSecurityValueHolders(unittest.TestCase):
         for name in expected:
             self.assertEqual(set(expected[name]), set(calculated[name]))
 
+    def testDependencyCalculationOnCompoundedValueHolder(self):
+        h = SecurityMovingMax(5, SecurityMovingAverage(10, 'close', ['aapl']) + SecurityMovingAverage(20, 'open', ['aapl']))
+        expected = {'aapl': ['close', 'open']}
+        calculated = h.dependency
+        for name in expected:
+            self.assertEqual(set(calculated[name]), set(expected[name]))
+
     def testItemizedValueHolder(self):
         window = 10
         pNames = 'close'
         symbolList = ['AAPL', 'IBM', 'GOOG']
         test = SecurityMovingAverage(window, pNames, symbolList)
+        test.push({'aapl': {'close': 10.0}, 'ibm': {'close': 15.0}, 'goog': {'close': 17.0}})
+        test.push({'aapl': {'close': 12.0}, 'ibm': {'close': 10.0}, 'goog': {'close': 13.0}})
 
         # single named value holder
-        test1 = test['IBM']
-        self.assertEqual(set(test1.symbolList), set(['ibm']))
-        self.assertEqual(test1.dependency, {'ibm': pNames})
-        self.assertEqual(test1.valueSize, 1)
-        self.assertEqual(test1.window, window)
+        test1 = test['ibm']
+        self.assertAlmostEqual(test1, 12.5, 15)
 
         # multi-valued named value holder
-        test2 = test['IBM', 'GOOG']
-        dependency = {
-            name: pNames for name in ['ibm', 'goog']
-            }
-        self.assertTrue(isinstance(test2, SecurityValueHolder))
-        self.assertEqual(set(test2.symbolList), set(['ibm', 'goog']))
-        self.assertEqual(test2.dependency, dependency)
-        self.assertEqual(test2.valueSize, 1)
-        self.assertEqual(test2.window, window)
+        test2 = test['ibm', 'goog']
+        expected = SecuritiesValues({'ibm': 12.5, 'goog':15.0})
+        for s in test2.index:
+            self.assertAlmostEqual(test2[s], expected[s])
 
         # wrong type of item
         with self.assertRaises(TypeError):
@@ -257,7 +251,7 @@ class TestSecurityValueHolders(unittest.TestCase):
 
             expected = ma.value + mm.value
             calculated = combined.value
-            for name in expected:
+            for name in expected.index:
                 self.assertAlmostEqual(expected[name], calculated[name], 12)
 
     def testAddedSecurityValueHoldersWithScalar(self):
@@ -275,7 +269,7 @@ class TestSecurityValueHolders(unittest.TestCase):
 
             expected = 2.0 + mm.value
             calculated = combined.value
-            for name in expected:
+            for name in expected.index:
                 self.assertAlmostEqual(expected[name], calculated[name], 12)
 
     def testRAddedSecurityValueHoldersWithScalar(self):
@@ -293,7 +287,7 @@ class TestSecurityValueHolders(unittest.TestCase):
 
             expected = ma.value + 2.0
             calculated = combined.value
-            for name in expected:
+            for name in expected.index:
                 self.assertAlmostEqual(expected[name], calculated[name], 12)
 
     def testSubbedSecurityValueHolder(self):
@@ -316,7 +310,7 @@ class TestSecurityValueHolders(unittest.TestCase):
 
             expected = ma.value - mm.value
             calculated = combined.value
-            for name in expected:
+            for name in expected.index:
                 self.assertAlmostEqual(expected[name], calculated[name], 12)
 
     def testSubbedSecurityValueHoldersWithScalar(self):
@@ -334,7 +328,7 @@ class TestSecurityValueHolders(unittest.TestCase):
 
             expected = 2.0 - mm.value
             calculated = combined.value
-            for name in expected:
+            for name in expected.index:
                 self.assertAlmostEqual(expected[name], calculated[name], 12)
 
     def testRSubbedSecurityValueHoldersWithScalar(self):
@@ -352,7 +346,7 @@ class TestSecurityValueHolders(unittest.TestCase):
 
             expected = ma.value - 2.0
             calculated = combined.value
-            for name in expected:
+            for name in expected.index:
                 self.assertAlmostEqual(expected[name], calculated[name], 12)
 
     def testMultipliedSecurityValueHolders(self):
@@ -375,7 +369,7 @@ class TestSecurityValueHolders(unittest.TestCase):
 
             expected = ma.value * mm.value
             calculated = combined.value
-            for name in expected:
+            for name in expected.index:
                 self.assertAlmostEqual(expected[name], calculated[name], 12)
 
     def testMultipliedSecurityValueHoldersWithScalar(self):
@@ -393,7 +387,7 @@ class TestSecurityValueHolders(unittest.TestCase):
 
             expected = 2.0 * mm.value
             calculated = combined.value
-            for name in expected:
+            for name in expected.index:
                 self.assertAlmostEqual(expected[name], calculated[name], 12)
 
     def testRMultipliedSecurityValueHoldersWithScalar(self):
@@ -411,7 +405,7 @@ class TestSecurityValueHolders(unittest.TestCase):
 
             expected = ma.value * 2.0
             calculated = combined.value
-            for name in expected:
+            for name in expected.index:
                 self.assertAlmostEqual(expected[name], calculated[name], 12)
 
     def testDividedSecurityValueHolders(self):
@@ -434,7 +428,7 @@ class TestSecurityValueHolders(unittest.TestCase):
 
             expected = ma.value / mm.value
             calculated = combined.value
-            for name in expected:
+            for name in expected.index:
                 self.assertAlmostEqual(expected[name], calculated[name], 12)
 
     def testDividedSecurityValueHoldersWithScalar(self):
@@ -452,7 +446,7 @@ class TestSecurityValueHolders(unittest.TestCase):
 
             expected = 2.0 / mm.value
             calculated = combined.value
-            for name in expected:
+            for name in expected.index:
                 self.assertAlmostEqual(expected[name], calculated[name], 12)
 
     def testRDividedSecurityValueHoldersWithScalar(self):
@@ -470,18 +464,15 @@ class TestSecurityValueHolders(unittest.TestCase):
 
             expected = ma.value / 2.0
             calculated = combined.value
-            for name in expected:
+            for name in expected.index:
                 self.assertAlmostEqual(expected[name], calculated[name], 12)
 
     def testCombinedSecurityValueHolderWithoutData(self):
         ma = SecurityMovingAverage(10, 'close', ['aapl', 'ibm', 'goog'])
-        expected = {'aapl': np.nan,
-                    'ibm': np.nan,
-                    'goog': np.nan}
         calculated = ma.value
 
-        for name in expected:
-            self.assertIs(expected[name], calculated[name])
+        for name in calculated.index:
+            self.assertTrue(np.isnan(calculated[name]))
 
     def testShiftedSecurityValueHolder(self):
         mm = SecurityMovingAverage(2, 'close', ['aapl', 'ibm', 'goog'])
@@ -491,33 +482,32 @@ class TestSecurityValueHolders(unittest.TestCase):
                  'ibm': {'close': 2.0},
                  'goog': {'close': 3.0}}
         shifted1.push(data1)
-        expected = {'aapl': np.nan,
-                    'ibm': np.nan,
-                    'goog': np.nan}
         calculated = shifted1.value
-        for name in expected:
-            self.assertIs(expected[name], calculated[name])
+        for name in calculated.index:
+            self.assertTrue(np.isnan(calculated[name]))
 
         data2 = {'aapl': {'close': 2.0},
                  'ibm': {'close': 3.0},
                  'goog': {'close': 4.0}}
+
         shifted1.push(data2)
-        expected = {'aapl': 1.0,
+        expected = SecuritiesValues({'aapl': 1.0,
                    'ibm': 2.0,
-                   'goog': 3.0}
+                   'goog': 3.0})
         calculated = shifted1.value
-        for name in expected:
+        for name in expected.index:
             self.assertAlmostEqual(expected[name], calculated[name])
 
-        data3 = {'aapl': {'close': 3.0},
+        data3 = SecuritiesValues({'aapl': {'close': 3.0},
                  'ibm': {'close': 4.0},
-                 'goog': {'close': 5.0}}
+                 'goog': {'close': 5.0}})
+
         shifted1.push(data3)
-        expected = {'aapl': 1.5,
+        expected = SecuritiesValues({'aapl': 1.5,
                     'ibm': 2.5,
-                    'goog': 3.5}
+                    'goog': 3.5})
         calculated = shifted1.value
-        for name in expected:
+        for name in expected.index:
             self.assertAlmostEqual(expected[name], calculated[name])
 
     def testShiftedSecurityValueHolderWithLengthZero(self):
@@ -536,17 +526,110 @@ class TestSecurityValueHolders(unittest.TestCase):
                     'ibm': {Factors.CLOSE: self.datas['ibm'][Factors.CLOSE][i]}}
             ma.push(data)
             maRes = ma.value
-            for name in maRes:
+            for name in maRes.index:
                 container[name].append(maRes[name])
                 expected[name] = max(container[name])
 
             compounded.push(data)
             calculated = compounded.value
-            for name in calculated:
+            for name in calculated.index:
                 self.assertAlmostEqual(expected[name], calculated[name], 12, "for {0} at index {1}\n"
                                                                              "expected:   {2}\n"
                                                                              "calculated: {3}"
                                        .format(name, i, expected[name], calculated[name]))
 
+    def testLtSecurityValueHolder(self):
+        filter = SecurityLatestValueHolder('close', ['aapl', 'ibm', 'goog']) < 10.0
+        ma = SecurityMovingAverage(10, 'close', ['aapl', 'ibm', 'goog'])[filter]
 
+        data = {'aapl': {'close': 15.},
+                'ibm': {'close': 8.},
+                'goog': {'close': 7.}}
 
+        ma.push(data)
+        expected = {'ibm': 8.0, 'goog': 7.0}
+        calculated = ma.value
+        for name in expected:
+            self.assertAlmostEqual(expected[name], calculated[name], 15)
+
+        data = {'aapl': {'close': 9.},
+                'ibm': {'close': 11.},
+                'goog': {'close': 8.}}
+
+        ma.push(data)
+        expected = {'aapl': 12.0, 'goog': 7.5}
+        calculated = ma.value
+        for name in expected:
+            self.assertAlmostEqual(expected[name], calculated[name], 15)
+
+    def testLeSecurityValueHolder(self):
+        filter = SecurityLatestValueHolder('close', ['aapl', 'ibm', 'goog']) <= 10.0
+        ma = SecurityMovingAverage(10, 'close', ['aapl', 'ibm', 'goog'])[filter]
+
+        data = {'aapl': {'close': 15.},
+                'ibm': {'close': 10.},
+                'goog': {'close': 7.}}
+
+        ma.push(data)
+        expected = {'ibm': 10.0, 'goog': 7.0}
+        calculated = ma.value
+        for name in expected:
+            self.assertAlmostEqual(expected[name], calculated[name], 15)
+
+        data = {'aapl': {'close': 10.},
+                'ibm': {'close': 11.},
+                'goog': {'close': 8.}}
+
+        ma.push(data)
+        expected = {'aapl': 12.5, 'goog': 7.5}
+        calculated = ma.value
+        for name in expected:
+            self.assertAlmostEqual(expected[name], calculated[name], 15)
+
+    def testGtSecurityValueHolder(self):
+        filter = SecurityLatestValueHolder('close', ['aapl', 'ibm', 'goog']) > 10.0
+        ma = SecurityMovingAverage(10, 'close', ['aapl', 'ibm', 'goog'])[filter]
+
+        data = {'aapl': {'close': 15.},
+                'ibm': {'close': 8.},
+                'goog': {'close': 7.}}
+
+        ma.push(data)
+        expected = {'aapl': 15.}
+        calculated = ma.value
+        for name in expected:
+            self.assertAlmostEqual(expected[name], calculated[name], 15)
+
+        data = {'aapl': {'close': 9.},
+                'ibm': {'close': 11.},
+                'goog': {'close': 8.}}
+
+        ma.push(data)
+        expected = {'ibm': 9.5}
+        calculated = ma.value
+        for name in expected:
+            self.assertAlmostEqual(expected[name], calculated[name], 15)
+
+    def testGeSecurityValueHolder(self):
+        filter = SecurityMovingAverage(1, 'close', ['aapl', 'ibm', 'goog']) >= 10.0
+        ma = SecurityMovingAverage(10, 'close', ['aapl', 'ibm', 'goog'])[filter]
+
+        data = {'aapl': {'close': 15.},
+                'ibm': {'close': 10.},
+                'goog': {'close': 7.}}
+
+        ma.push(data)
+        expected = {'aapl': 15., 'ibm': 10.}
+        calculated = ma.value
+        for name in expected:
+            self.assertAlmostEqual(expected[name], calculated[name], 15)
+
+        data = {'aapl': {'close': 10.},
+                'ibm': {'close': 11.},
+                'goog': {'close': 8.}}
+
+        ma.push(data)
+        expected = {'aapl': 12.5, 'ibm': 10.5}
+        calculated = ma.value
+        for name in expected:
+            self.assertAlmostEqual(expected[name], calculated[name], 15)
